/*
    This file is part of the 'yarec' package.
    Copyright (C) 1998  Ralf W. Stephan <stephan@tmt.de>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
/* Special thanks to the aumix authors (see THANKS) for the ncurses code */

#include "uiHandler.h"

#ifdef UI_CURSES

#if defined HAVE_STRING_H || defined _LIBC
# include <string.h>
#else
# include <strings.h>
#endif

#include <ctype.h>
#include <curses.h>
#include "util/util.h"
#include "tasks/tasks.h"

static void init_colors();
static void init_screen();
static void status (const char*);
static void time (const char*);
static void size (long, long);
static void rbc (int, int, int);
static void hide();
static int get_key();
static void task_play(), task_record();

static WINDOW* win_;
static int kbd_buf_ = -1, inited_ = 0;

void CursesUI_init (int* ip, char ***cp)
{
  win_ = initscr();
  noecho();
  leaveok (stdscr, TRUE);
  nodelay (stdscr, TRUE);
  curs_set (0);
  inited_ = 1;
}

int CursesUI_is_commandline()
{
  return 1;
}

void CursesUI_print_to_console (const char* s1, const char* s2)
{
  if (!s2) s2 = "";
  fprintf (stderr, "%s %s\n", s1, s2);
  fflush (stderr);
}

void CursesUI_draw_box()
{
  init_colors();
  init_screen();
}

static void shutdown() { Exception_normal ("curses done"); }

void CursesUI_takes_over()
{
  sigpipe_handler (shutdown);
  sigterm_handler (shutdown);
  sigint_handler (shutdown);

  if (!Config_iget ("RECORDING")) {
	task_play();
  	Exception_normal ("Done playing.");
  	/* NOT REACHED */
  }
  
  if (Config_iget ("FORCE_ACTION")) {
  	task_record();
  	Task_write();
  	Exception_normal ("Done recording/saving.");
  	/* NOT REACHED */
  }
  
  do {
  	int c;
  	
  	if (kbd_buf_ == -1)
  		c = get_key();
  	else {
  		c = kbd_buf_;
  		kbd_buf_ = -1;
  	}
  		
  	c = tolower (c);
  	
  	switch (c) {
  	case 'p':
  		task_play();
  		break;
  	case 'r':
  	case '\n':
  		task_record();
  		break;
  	case 'w':
  		Task_write();
  		break;
  	case 'q':
  		Exception_normal ("Done.");
  		break;
  	default:
  		beep();
  		/*fprintf (stderr, "Unsupported key: %c\n", c);*/
  	}
  } while (1);
}

void CursesUI_msg (const char* m)
{
  status (m);
}

void CursesUI_time (const char* s)
{
  time (s);
}

void CursesUI_size (long s1, long s2)
{
  size (s1, s2);
}

void CursesUI_rbc (int r, int b, int c)
{
  rbc (r, b, c);
}

void CursesUI_close()
{
  if (!inited_) return;
  clear();
  refresh();
  endwin();
}

//--------------------------LOW-LEVEL STUFF----------------------------
#define TITLE_COLOR 1
#define TITLE_COLOR2 2
#define HANDLE_COLOR 3
#define TRACK_COLOR 4
#define REC_LED_COLOR 5
#define PLAY_LED_COLOR 6
#define ACTIVE_COLOR 7
#define INACTIVE_COLOR 8
#define INCREMENT 4
#define XOFFSET 9
#define YOFFSET 0

static WINDOW* w_panel;

static void init_colors()
{
    start_color();
    init_pair(TITLE_COLOR, COLOR_CYAN, COLOR_BLUE);
    init_pair(TITLE_COLOR2, COLOR_RED, COLOR_BLUE);
    init_pair(HANDLE_COLOR, COLOR_CYAN, COLOR_BLUE);
    init_pair(TRACK_COLOR, COLOR_BLUE, COLOR_BLACK);
    init_pair(REC_LED_COLOR, COLOR_RED, COLOR_BLACK);
    init_pair(PLAY_LED_COLOR, COLOR_GREEN, COLOR_BLACK);
    init_pair(ACTIVE_COLOR, COLOR_YELLOW, COLOR_RED);
    init_pair(INACTIVE_COLOR, COLOR_WHITE, COLOR_BLACK);
}

static void init_screen()
{
    w_panel = newwin(LINES - 1, COLS, 0, 0);
    wclear(w_panel);
    wattrset(w_panel, COLOR_PAIR(TITLE_COLOR));
    wmove(w_panel, 2, 1);
    waddstr(w_panel, "uit   ");
    wmove(w_panel, 3, 1);
    waddstr(w_panel, "lay   ");
    wmove(w_panel, 4, 1);
    waddstr(w_panel, "ecord ");
    wmove(w_panel, 5, 1);
    waddstr(w_panel, "alt   ");
    wmove(w_panel, 6, 1);
    waddstr(w_panel, "top   ");
    wmove(w_panel, 7, 1);
    waddstr(w_panel, "rite  ");
    wattrset(w_panel, COLOR_PAIR(TITLE_COLOR) | A_UNDERLINE);
    wmove(w_panel, 0, 0);
    waddstr(w_panel, " yarec ");
    wattrset(w_panel, COLOR_PAIR(TITLE_COLOR2) | A_BOLD);
    wmove(w_panel, 2, 0);
    waddstr(w_panel, "Q");
    wmove(w_panel, 3, 0);
    waddstr(w_panel, "P");
    wmove(w_panel, 4, 0);
    waddstr(w_panel, "R");
    wmove(w_panel, 5, 0);
    waddstr(w_panel, "H");
    wmove(w_panel, 6, 0);
    waddstr(w_panel, "S");
    wmove(w_panel, 7, 0);
    waddstr(w_panel, "W");

    wattrset(w_panel, COLOR_PAIR(INACTIVE_COLOR));
    wmove(w_panel, 2, 9);
    waddstr(w_panel, "Sampling rate  :");
    wmove(w_panel, 3, 9);
    waddstr(w_panel, "Bits per sample:");
    wmove(w_panel, 4, 9);
    waddstr(w_panel, "n of channels  :");
    wmove(w_panel, 5, 9);
    waddstr(w_panel, "File format    :");
    wmove(w_panel, 6, 9);
    waddstr(w_panel, "File basename  :");
    
    wmove(w_panel, 2, 26);
    waddstr(w_panel, Config_sget ("SOUND_SPEED"));
    wmove(w_panel, 3, 26);
    waddstr(w_panel, Config_sget ("SOUND_BITS"));
    wmove(w_panel, 4, 26);
    waddstr(w_panel, Config_sget ("SOUND_CHANNELS"));
    wmove(w_panel, 5, 26);
    waddstr(w_panel, Config_sget ("FILE_FORMAT"));
    wmove(w_panel, 6, 26);
    waddstr(w_panel, Config_sget ("BASENAME"));

    wmove(w_panel, 2, 33);
    waddstr(w_panel, "Time :");
    wmove(w_panel, 3, 33);
    waddstr(w_panel, "Size :");
    wmove(w_panel, 2, 50);
    waddstr(w_panel, "Other keys : RETURN for R");
    wmove(w_panel, 3, 50);
    waddstr(w_panel, "             SPACE  for S");

    hide();    
    refresh();
    wrefresh(w_panel);
}

static void status (const char* s)
{
  char buf[71];
  strncpy (buf, s, 70);
  buf[70] = '\0';
  wmove (w_panel, 0, 8);
  wclrtoeol (w_panel);
  wmove (w_panel, 0, 9);
  waddstr (w_panel, buf);
  hide();
  refresh();
  wrefresh(w_panel);
}

static void time (const char* s)
{
  char buf[9], *clear = "        ";
  strncpy (buf, s, 8);
  buf[8] = '\0';
  wmove (w_panel, 2, 40);
  waddstr (w_panel, clear);
  wmove (w_panel, 2, 40);
  waddstr (w_panel, buf);
  hide();
  refresh();
  wrefresh(w_panel);
}

static void size (long s1, long s2)
{
  char buf[16], tmp[8];
  strcpy (tmp, l2si (s1));
  sprintf (buf, "%6s / %6s", tmp, l2si (s2));
  wmove (w_panel, 3, 40);
  waddstr (w_panel, buf);
  hide();
  refresh();
  wrefresh(w_panel);
}

static void rbc (int r, int b, int c)
{
  char buf[16];

  sprintf (buf, "%-6d", r);
  wmove(w_panel, 2, 26);
  waddstr(w_panel, buf);
  sprintf (buf, "%-2d", b);
  wmove(w_panel, 3, 26);
  waddstr(w_panel, buf);
  sprintf (buf, "%-2d", c);
  wmove(w_panel, 4, 26);
  waddstr(w_panel, buf);
  hide();
  refresh();
  wrefresh(w_panel);
}

static void hide()
{
  wmove(w_panel, LINES-2, COLS-1);
}

static int get_key() 
{
  int i;
  status ("");
  nodelay (stdscr, FALSE);
  i = getch();
  nodelay (stdscr, TRUE);
  return i;
}

static int stopped()
{
  int i = getch();
  kbd_buf_ = -1;
  if (i == ERR) return 0;
  i = tolower (i);
  
  switch (i) {
  case ' ':
  case 's': 
  	return 2;
  case 'q':
  	kbd_buf_ = i;
  	return 2;
  case 'h':
  	return 1;
  default:
  	beep();
  	return 0;
  }
  
  /* NOT REACHED */
  return -1;
}


static void task_play()
{
  int b=1, s;
  Task_schedule ('P');
  do {
  	napms (100);
  	s = stopped();
  	b = Task_timeout();
  }
  while (!s && b);
  Task_schedule ((s==2 || !b)? 'S':'H');
}

static void task_record()
{
  int b=1, s;
  Task_schedule ('R');
  do {
  	napms (100);
  	s = stopped();
  	b = Task_timeout();
  }
  while (!s && b);
  Task_schedule ((s==2 || !b)? 'S':'H');
}



#endif
